/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package bachelorprojectarticleinfo;

import app.bachelorproject.auth.Group;
import bachelorprojectlog.LogEntry;
import bachelorprojectstock.Stock;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.eclipse.microprofile.jwt.JsonWebToken;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author marti
 */
@Path("articles")
@Stateless
@DeclareRoles({Group.USER, Group.ADMIN})
public class ArticleInfoService {
    @Context
    SecurityContext sc;
    
    @PersistenceContext
    EntityManager em;
    
    @Inject
    JsonWebToken principal;
    
    @GET
    @Path("getarticlesforstock")
    @Produces(MediaType.APPLICATION_JSON)
    public Response getArticlesForStock(
            @QueryParam("stock")@NotBlank String stock){
       
        List<ArticleInfoEntity> allArticles = em.createNamedQuery("ArticleInfoEntity.findAllArticleInfo", ArticleInfoEntity.class).getResultList();
        ArrayList<ArticleInfoEntity> relevantArticles = new ArrayList<ArticleInfoEntity>();
        List<Stock> tempStocks;
        
        tempStocks = em.createNamedQuery("Stock.findByName", Stock.class)
                .setParameter("stockName", stock)
                .getResultList();
        System.out.println(tempStocks.get(0));
        
        ArticleInfoEntity currentArticle;
        for(int i = 0; i<allArticles.size(); i++){
            currentArticle = allArticles.get(i);
            System.out.println("name: " + currentArticle.stockName + " reference: " + stock);
            
            if (currentArticle.stockName.equals(stock)){
                relevantArticles.add(currentArticle);
                
            }
        }
        return Response.ok(relevantArticles).build();
    }
    
    
    @GET
    @Path("getallarticleinfo")
    public List getAllStocks(){
        return em.createNamedQuery("ArticleInfoEntity.findAllArticleInfo", ArticleInfoEntity.class).getResultList();
    }
    
    @GET
    @Path("getlastdate")
    public String getLastDate(){
        List<ArticleInfoEntity> dates = em.createNamedQuery("ArticleInfoEntity.findAllArticleInfo", ArticleInfoEntity.class).getResultList();
        
        Calendar newestDate;
        Calendar currentDate;
        String newestDateString = "1950-02-23";
        newestDate = StringToCalendar("1950-02-23");
        for(int i = 0; i < dates.size(); i++){
            currentDate = StringToCalendar(dates.get(i).date);
            

            if(newestDate.before(currentDate)){
                newestDate = currentDate;
                newestDateString = dates.get(i).date;
            }
            
        }
        return newestDateString;
    }
    
    @POST
    @Path("createwjson")
    @Consumes("application/json")
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response Bar(String json) throws JSONException{
        JSONArray ja = new JSONArray(json);
        ArrayList<ArticleInfoEntity> articleInfoEntities = new ArrayList<ArticleInfoEntity>();
        
        String url;
        String stockName;
        String title;
        String date;
        String publisher;
        List<String> negativeWords;
        List<String> positiveWords;
        
        int sentiment;
        
        for(int i = 0; i < ja.length(); i++){
            try{
                JSONObject jo = new JSONObject(ja.getString(i));
                System.out.print(jo);
                negativeWords = new ArrayList();
                positiveWords = new ArrayList();
            
                url = jo.getString("URL");
                stockName = jo.getString("Name");
                title = jo.getString("Title");
                date = jo.getString("Date");
                publisher = jo.getString("Publisher");
                sentiment = jo.getInt("Sentiment");
                JSONArray jaNegative = jo.getJSONArray("Negative");
                JSONArray jaPositive = jo.getJSONArray("Positive");
                
                for(int y = 0; y < jaNegative.length(); y++){
                    negativeWords.add(jaNegative.getString(y));
                    //System.out.println(y);
                }
                for(int y = 0; y < jaPositive.length(); y++){
                    positiveWords.add(jaPositive.getString(y));
                }
            
                ArticleInfoEntity referenceEntity = em.find(ArticleInfoEntity.class, url);
            
                if(referenceEntity == null){
                    ArticleInfoEntity entity = new ArticleInfoEntity(url, title, stockName, date, publisher, sentiment, negativeWords, positiveWords);
                    articleInfoEntities.add(entity);
                    em.persist(entity);
                    em.flush();
            
                }
            }catch(Exception e){
                
            }

        }

        return Response.ok(articleInfoEntities).build();
    }
    
    private Calendar StringToCalendar(String date){
        
        Calendar returnDate = Calendar.getInstance();
        returnDate.clear();
        
        int day = 0;
        int month = 0;
        int year = 0;
        
        String delims = "[-]";
        String[] tokens = date.split(delims);
        
                for (int i = 0; i<tokens.length; i++){
            tokens[i].replaceFirst("^0+(?!$)", "");
            if (i == 0){
                year = Integer.parseInt(tokens[i]);
                
            }
            if (i == 1){
                month = Integer.parseInt(tokens[i]);
                
            }
            if (i == 2){
                day = Integer.parseInt(tokens[i]);
                
            }
        }
        
        returnDate.set(year, month - 1, day);
        return returnDate;
    
        
    }
}
