/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package bachelorprojectfavourites;

import app.bachelorproject.auth.Group;
import app.bachelorproject.auth.User;
import bachelorprojectlog.LogEntry;
import bachelorprojectstock.Stock;
import java.util.List;
import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.eclipse.microprofile.jwt.JsonWebToken;

/**
 *
 * @author marti
 */
@Path("favouritestocks")
@Stateless
@DeclareRoles({Group.USER, Group.ADMIN})
public class FavouriteStockService {
        
    @Context
    SecurityContext sc;
    
    @PersistenceContext
    EntityManager em;
    
    @Inject
    JsonWebToken principal;
    
    @GET
    @Path("getfavourites")
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed({Group.ADMIN, Group.USER})
    public List getUserFavourites(){
        return em.createNamedQuery("FavouriteStockEntity.findFavouritesByUser", FavouriteStockEntity.class)
                .setParameter("userid", sc.getUserPrincipal().getName())
                .getResultList();
    }
    
    @POST
    @Path("create")
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response create(
            @FormParam("sid") String stockId){
        FavouriteStockEntity newFav;
        User user = em.find(User.class, sc.getUserPrincipal().getName());
        Stock tempStock = em.find(Stock.class, stockId);
        
        List<FavouriteStockEntity> usersFavourites = em.createNamedQuery("FavouriteStockEntity.findFavouritesByUser", FavouriteStockEntity.class)
                .setParameter("userid", sc.getUserPrincipal().getName())
                .getResultList();
        
        boolean alreadyExists = false;
        for (int i = 0; i < usersFavourites.size(); i++){
            FavouriteStockEntity temp = usersFavourites.get(i);
            if (temp.getStock().getStockId().equals(stockId)){
                alreadyExists = true;
            }
            
        }
        if(!alreadyExists){
            newFav = new FavouriteStockEntity(user, tempStock);
        
        em.persist(newFav);
        
        return Response.ok(newFav).build();
        }else{
            return Response.status(403, "Already Exists").build();
            
        }
    }
    
    @POST
    @Path("remove")
    @Produces(MediaType.APPLICATION_JSON)
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response remove(
            @FormParam("fid") String favouriteId){
        
        FavouriteStockEntity newFav = em.find(FavouriteStockEntity.class, Integer.parseInt(favouriteId));
        if(newFav == null){
            return Response.status(404, "No such entity").build();
        }else{
            em.remove(newFav);
            em.flush();
            return Response.ok().build();
        }
        
        
        
    }
}
