/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package bachelorprojectstock;

import app.bachelorproject.auth.Group;
import bachelorprojectlog.LogEntry;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.security.DeclareRoles;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.SecurityContext;
import org.eclipse.microprofile.jwt.JsonWebToken;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author marti
 */
@Path("stock")
@Stateless
@DeclareRoles({Group.USER, Group.ADMIN})
public class StockService {
    
    @Context
    SecurityContext sc;
    
    @PersistenceContext
    EntityManager em;
    
    @Inject
    JsonWebToken principal;
    
    @GET
    @Path("getstocks")
    public List getAllStocks(){
        return em.createNamedQuery("Stock.findAllStocks", Stock.class).getResultList();
    }
    
    @POST
    @Path("createwjson")
    @Consumes("application/json")
    @RolesAllowed({Group.ADMIN, Group.USER})
    public Response bar(String json) throws JSONException{
        LocalDateTime referenceTime = LocalDateTime.now();
        JSONArray ja = new JSONArray(json);
        ArrayList<Stock> stocks = new ArrayList<Stock>();
        
        String stockTag;
        String stockName;
        String stockMarket;
        String stockId;
        
        
        for(int i = 0; i < ja.length(); i++){
            try{
                JSONObject jo = new JSONObject(ja.getString(i));
                System.out.println(jo);
                stockTag = jo.getString("Tag");
                stockName = jo.getString("Name");
                stockMarket = jo.getString("Market");
                stockId = (stockTag + stockMarket);
                Stock referenceStock = em.find(Stock.class, stockId);
                
                //check if it exists and if not, persist to db
                if(referenceStock == null){
                    Stock stock = new Stock(stockId, stockTag, stockName, stockMarket);
                    stocks.add(stock);
                    em.persist(stock);
                    em.flush();
                }
                //if it exists, check if name is updated, update if it is
                //as of now name is the only thing we can change
                //update time, which causes stocks no longer imported to
                //get an old time, which is later found and deleted
                
                else{
                    if (!referenceStock.getName().equals(stockName)){
                        referenceStock.setName(stockName);
                    }
                    referenceStock.setLastChecked(LocalDateTime.now());
                }
                

            }catch(Exception e){
                
            }
            
        }
        
        
        List<Stock> allStocks = getAllStocks();
        //find and delete all stocks with outdated timestamps
        for(int i = 0; i < allStocks.size(); i++){
            Stock s = allStocks.get(i);
            if(s.getLastChecked().isBefore(referenceTime)){
                em.remove(em.find(Stock.class, s.getStockId()));
                em.flush();
            }
            
        }
        
        return Response.ok(stocks).build();
    }
    
    @GET
    @Path("getstock")
    @RolesAllowed({Group.USER, Group.ADMIN})
    public Response getStockName(@QueryParam("name") List<String> stockName){
        List<Stock> returnStocks = new ArrayList<>();
        for(int i = 0; i<stockName.size(); i++){
            
            List<Stock> stocks = em.createNamedQuery("Stock.findByTag", Stock.class)
                .setParameter("stockTag", stockName.get(i))
                .getResultList();
            System.out.println(stocks);
            returnStocks.add(stocks.get(0));
        }
        return Response.ok(returnStocks).build();
        
        
    }
}
