#!/lib/sh
from selenium import webdriver
from selenium.common.exceptions import TimeoutException, NoSuchElementException
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.chrome.options import Options
from time import sleep
import requests
import json
import re
from datetime import datetime as dt
import simplesentiment as sentiments
from publisher import Publisher
from article import Article
import time
import traceback
import sys
import os
import fnmatch

sleep(60)


# Gets the last date the script ran from the server
def getLastRunDate():
    APIURLGETDATE = "http://158.38.101.196:9000/BachelorTestOne-1.0/api/articles/getlastdate"
    headers = {'Authorization': 'Bearer ' + token}
    past = requests.get(APIURLGETDATE).text
    return past


def logIn():
    APIURLLOGIN = "http://158.38.101.196:9000/BachelorTestOne-1.0/api/auth/login"
    data = {'uid': 'crawleruser',
            'pwd': 'crawlerpassword'}
    past = requests.post(APIURLLOGIN, data=data)
    return past.text


# Gets Json array from server containing stock names
def getJsonFromServer():
    APIURLGET = "http://158.38.101.196:9000/BachelorTestOne-1.0/api/stock/getstocks"

    headers = {'Content-Type': 'application/x-www-form-urlencoded;Charset=UTF-8',
               'Authorization': 'Bearer ' + token}

    stocknamerequest = requests.get(APIURLGET)
    data = json.loads(stocknamerequest.text)
    for name in data:
        stocknames.append(name['name'])
        print(name)
    stocknames.append("OSLO BØRS")
    #print("STOCK NAMES: " + stocknames)


# Sends Json array to server
def sendJsonToServer():
    APIURLPOST = "http://158.38.101.196:9000/BachelorTestOne-1.0/api/articles/createwjson"
    headers = {'Content-Type': 'application/json;Charset=UTF-8',
               'Authorization': 'Bearer ' + token}
    # requests.post(APIURLPOST, json=jsonObjects)
    articlerequest = requests.post(APIURLPOST, json=jsonObjects, headers=headers)
    return articlerequest


# Changes the string words from a single string to a single string with space between words
def listToString(words):
    list = str.split(words)
    str1 = ""
    for ele in list:
        str1 += ele
        str1 += " "
    return str1


# Converts the dates collected from website and server
def dateConverter(date, type):
    if type == "past":
        converteddate = dt.strptime(date, '%Y-%m-%d')
    else:
        converteddate = dt.strptime(date, '%d.%m.%Y')
    converteddate = converteddate.date()
    return converteddate


# Creates a jsonobject from the Article object
def jsonCreater(article):
    data_set = {"Name": article.name, "Title": article.title, "URL": str(article.url),
                "Date": str(article.date), "Publisher": article.publisher, "Sentiment": article.sentiment,
                "Negative": article.listOfNegativeWords,
                "Positive": article.listOfPositiveWords}
    jsonObjects.append(data_set)
    json_dump = json.dumps(jsonObjects, ensure_ascii=False).encode('utf8')
    print(json_dump.decode())


# Creates a Object that contains all locations needed for the wbcrawler to search and scrape the website
def getURLfrompublisher(publisherName):
    global publisherInfo
    if publisherName == "dagens næringsliv":
        url = 'https://www.dn.no/'
        searchbtn = '/html/body/div[2]/header/div/div/div[1]/div[2]/nav/ul/li[5]/div/div[1]/a'
        searchfld = '/html/body/div[2]/header/div/div/div[1]/div[2]/nav/ul/li[5]/div/div[2]/div/div/div/div/div/form/div/input'
        executesearch = '/html/body/div[2]/header/div/div/div[1]/div[2]/nav/ul/li[5]/div/div[2]/div/div/div/div/div/form/div/button'
        nrofresults = '/html/body/main/div[3]/section/header/div/h2'
        datelocation = '/html/body/main/div[3]/section/section/article[{}]/div/time'
        articleurllocation = '/html/body/main/div[3]/section/section/article[{}]/div/div/h3/a'
        articletitlelocation = '/html/body/main/div[3]/section/section/article[{}]/div/div/h3/a'
        articleContent = '/html/body/main/article/section/div[3]/p[{}]'
        signInButtonLocation = "/html/body/div[2]/header/div/div/div[2]/nav/ul/li/div/div[1]/a/span"
        usernameFieldLocation = "/html/body/main/section/div[1]/div[1]/div/div[1]/form/span[1]/input"
        passwordFieldLocation = "/html/body/main/section/div[1]/div[1]/div/div[1]/form/span[2]/input"
        executeSignInLocation = "/html/body/main/section/div[1]/div[1]/div/div[1]/form/button[1]"
        publisherInfo = Publisher(url, searchbtn, searchfld, executesearch, nrofresults, datelocation,
                                  articleurllocation, articletitlelocation, articleContent, signInButtonLocation,
                                  usernameFieldLocation, passwordFieldLocation, executeSignInLocation)
    else:
        print("failed to create publisher object")


# Convert a string containing the number of articles from a search to a int
def searchResultConverter(nrofarticles):
    results = str(nrofarticles.replace(".", ""))
    listofresults = re.findall('\d+', results)
    nrofresults = int(listofresults[0])
    return nrofresults


# The webcrawler extracts username and password from a text file and signs in to dagens næringsliv
def signIn():
    userName = ""
    password = ""
    # detect the current working directory
    path = os.getcwd()

    # read the entries
    with os.scandir(path) as listOfEntries:
        for entry in listOfEntries:
            # print all entries that are files
            if entry.is_file():
                print(entry.name)

    try:
        for word in open("/app/config", "r"):
            user = word.rstrip("\n")
            words = str.split(user, ":")
            if words[0] == "Username":
                userName = words[1]
            elif words[0] == "Password":
                password = words[1]
    except:
        # printing stack trace
        traceback.print_exception(*sys.exc_info())
  
    signInButton = driver.find_element_by_xpath(publisherInfo.signInButton)
    if signInButton.text == "Logg inn":
        signInButton.click()
        time.sleep(3)
        usernameField = driver.find_element_by_xpath(publisherInfo.usernameField)
        usernameField.send_keys(userName)
        passwordField = driver.find_element_by_xpath(publisherInfo.passwordField)
        passwordField.send_keys(password)
        executeSignIn = driver.find_element_by_xpath(publisherInfo.executeSignInButton)
        driver.execute_script("arguments[0].click();", executeSignIn)
    else:
        print("was already signed in")


# Returns the browser to homepage and searches for a new stock, returns number of articles found
def searchsite(i):
    driver.get(publisherInfo.url)
    time.sleep(3)
    signIn()
    # search
    searchbutton = driver.find_element_by_xpath(publisherInfo.searchButton)
    searchbutton.click()
    searchfield = driver.find_element_by_xpath(publisherInfo.searchField)
    searchfield.send_keys(stocknames[i])
    executesearch = driver.find_element_by_xpath(publisherInfo.executeSearch)
    executesearch.click()
    time.sleep(3)
    nrStringOfResults = driver.find_element_by_xpath(publisherInfo.nrOfResults).get_property(
        'innerHTML')
    nrOfResults = searchResultConverter(nrStringOfResults)
    return nrOfResults



# Enters article site and collects and returns the content in one string
def getCompleteArticle(articleURL):
    print(articleURL)
    driver.get(articleURL)
    time.sleep(3)
    x = 1
    i = 2
    completedArticle = ""
    while x < i:
        try:
            articlecont = publisherInfo.content.format(x)
            articleText = driver.find_element_by_xpath(articlecont).text
            completedArticle += articleText + " "
            i += 1
            x += 1
        except:
            x += 1
    driver.execute_script("window.history.go(-1)")
    return completedArticle


# Crawls the site
def crawlsite(publisher):
    global nrStringOfResults
    getURLfrompublisher(publisher)
    try:
        loadtries = 0
        maxtries = 5
        y = 0
        while y < len(stocknames):
            i = y
            article = Article(stocknames[i], publisher)
            nrOfResults = ""
            try:
                nrOfResults = searchsite(i)
            except:
                if loadtries <= maxtries:
                    loadtries = loadtries + 1
                    print("Element not interactable, trying again")
                    continue
                else:
                    print("Did not reach elements for:" + stocknames[i])
                    loadtries = 0
                    y += 1
            else:
                loadtries = 0
            pastdate = dateConverter(past, "past")
            print("Past Date: ")
            print(pastdate)
            presentdate = dt.today().date()
            y += 1
            x = 1
            while (x < nrOfResults) & (pastdate < presentdate) & (x < 5):
                dateLocation = publisherInfo.date.format(x)
                articleUrlLocation = publisherInfo.urlLocation.format(x)
                articleTitleLocation = publisherInfo.title.format(x)
                try:
                    datePublished = driver.find_element_by_xpath(dateLocation).get_attribute('datetime')
                    articleUrl = driver.find_element_by_xpath(articleUrlLocation).get_attribute('href')
                    articleTitle = driver.find_element_by_xpath(articleTitleLocation).get_attribute('text')
                    article.__set_date__(dateConverter(datePublished, " "))
                    article.__set_url__(articleUrl)
                    article.__set_title__(listToString(articleTitle))
                    content = getCompleteArticle(articleUrl)
                    print(content)
                    try:
                        sentiment, positiveWords, negativeWords = sentiments.sentimentAnalysis(content)
                        article.__set_sentiment_positive_negative__(sentiment, positiveWords, negativeWords)
                    except:
                        traceback.print_exception(*sys.exc_info())
                    
                except NoSuchElementException:
                    print("Date,URL or title not found")
                    x = x + 1
                    pass
                else:
                    presentdate = dateConverter(datePublished, " ")
                    if pastdate < presentdate:
                        jsonCreater(article)
                    else:
                        break
                    x = x + 1
        driver.close()
        sendJsonToServer()
    except TimeoutException:
        driver.close()
        print("failed to load")
    except NoSuchElementException:
        driver.close()
        print("Date,URL or title not found")
    except IndexError:
        driver.close()
        print("Search result of " + stocknames[i] + " did not load")
    except:
        driver.close()
        print("unknown error")


# Sets required variables, options for the browser and runs the webcrawler
def main():
    global stocknames
    global jsonObjects
    global driver
    global token
    global data
    global past
    stocknames = []
    jsonObjects = []
    chrome_options = Options()
    chrome_options.add_argument('--disable-dev-shm-usage')
    chrome_options.add_argument("--headless")
    driver = webdriver.Remote('http://selenium:4444/wd/hub', desired_capabilities=DesiredCapabilities.CHROME,
                              options=chrome_options)
    token = logIn()
    getJsonFromServer()
    past = getLastRunDate()
    publishername = "dagens næringsliv"
    crawlsite(publishername)


token = ""
publisherInfo = None
stocknames = []
jsonObjects = []
data = []
main()
